@@ -190,6 +190,9 @@ knit = function(
     knit_concord$set(infile = input, outfile = output)
   }
 
+  # we need some special treatment for chunks in Quarto document
+  .knitEnv$is_quarto = !is.null(opts_knit$get('quarto.version')) || ext == 'qmd'
+
   text = if (is.null(text)) xfun::read_utf8(input) else split_lines(text)
   if (!length(text)) {
     if (is.character(output)) file.create(output)
@@ -285,9 +288,13 @@ process_file = function(text, output) {
   # when in R CMD check, turn off the progress bar (R-exts said the progress bar
   # was not appropriate for non-interactive mode, and I don't want to argue)
   progress = opts_knit$get('progress') && !is_R_CMD_check()
+  labels = unlist(lapply(groups, function(g) {
+    if (is.list(g$params)) g[[c('params', 'label')]] else ''
+  }))
   if (progress) {
-    pb = txtProgressBar(0, n, char = '.', style = 3)
-    on.exit(close(pb), add = TRUE)
+    pb_fun = getOption('knitr.progress.fun', txt_pb)
+    pb = if (is.function(pb_fun)) pb_fun(n, labels)
+    on.exit(if (is.function(pb$done)) pb$done(), add = TRUE)
   }
   wd = getwd()
   for (i in 1:n) {
@@ -299,20 +306,21 @@ process_file = function(text, output) {
       }
       break  # must have called knit_exit(), so exit early
     }
-    if (progress) {
-      setTxtProgressBar(pb, i)
-      if (!tangle) cat('\n')  # under tangle mode, only show one progress bar
-      flush.console()
-    }
+    if (progress && is.function(pb$update)) pb$update(i)
     group = groups[[i]]
     res[i] = withCallingHandlers(
-      if (tangle) process_tangle(group) else process_group(group),
+      withCallingHandlers(
+        if (tangle) process_tangle(group) else process_group(group),
+        # TODO: remove the learnr hack https://github.com/rstudio/learnr/pull/781
+        error = function(e) if (xfun::pkg_available('rlang', '1.0.0') && !xfun::check_old_package('learnr', '0.11.3')) rlang::entrace(e)
+      ),
       error = function(e) {
         setwd(wd)
-        cat(res, sep = '\n', file = output %n% '')
+        write_utf8(res, output %n% stdout())
         message(
-          'Quitting from lines ', paste(current_lines(i), collapse = '-'),
-          ' (', knit_concord$get('infile'), ') '
+          '\nQuitting from lines ', paste(current_lines(i), collapse = '-'),
+          if (labels[i] != '') sprintf(' [%s]', labels[i]),
+          sprintf(' (%s)', knit_concord$get('infile'))
         )
       }
     )
@@ -331,7 +339,7 @@ auto_out_name = function(input, ext = tolower(file_ext(input))) {
   base = sans_ext(input)
   name = if (opts_knit$get('tangle')) c(base, '.R') else
     if (ext %in% c('rnw', 'snw')) c(base, '.tex') else
-      if (ext %in% c('rmd', 'rmarkdown', 'rhtml', 'rhtm', 'rtex', 'stex', 'rrst', 'rtextile'))
+      if (ext %in% c('rmd', 'rmarkdown', 'qmd', 'rhtml', 'rhtm', 'rtex', 'stex', 'rrst', 'rtextile'))
         c(base, '.', substring(ext, 2L)) else
           if (grepl('_knit_', input)) sub('_knit_', '', input) else
             if (ext != 'txt') c(base, '.txt') else c(base, '-out.', ext)
@@ -343,7 +351,7 @@ ext2fmt = c(
   rnw = 'latex', snw = 'latex', tex = 'latex', rtex = 'latex', stex = 'latex',
   htm = 'html', html = 'html', rhtml = 'html', rhtm = 'html',
   md = 'markdown', markdown = 'markdown', rmd = 'markdown', rmarkdown = 'markdown',
-  brew = 'brew', rst = 'rst', rrst = 'rst'
+  qmd = 'markdown', brew = 'brew', rst = 'rst', rrst = 'rst'
 )
 
 auto_format = function(ext) {
@@ -461,6 +469,20 @@ sew.character = function(x, options, ...) {
   knit_hooks$get('output')(x, options)
 }
 
+asis_token = '<!-- KNITR_ASIS_OUTPUT_TOKEN -->'
+wrap_asis = function(x, options) {
+  # do nothing when inside quarto as it is not needed
+  # https://github.com/yihui/knitr/pull/2212#pullrequestreview-1292924523
+  if (is_quarto()) return(x)
+
+  x = as.character(x)
+  if ((n <- length(x)) == 0 || !out_format('markdown') || missing(options) || !isTRUE(options$collapse))
+    return(x)
+  x[1] = paste0(asis_token, x[1])
+  x[n] = paste0(x[n], asis_token)
+  x
+}
+
 # If you provide a custom print function that returns a character object of
 # class 'knit_asis', it will be written as is.
 #' @export
@@ -478,10 +500,15 @@ sew.knit_asis = function(x, options, inline = FALSE, ...) {
     if (inherits(x, 'knit_asis_htmlwidget')) {
       options$fig.cur = plot_counter()
       options = reduce_plot_opts(options)
-      return(add_html_caption(options, x))
+      # look for attribute 'aria-labelledby="label"' in the first HTML tag and
+      # use the label to provide alt text if found
+      return(add_html_caption(
+        options, wrap_asis(x, options),
+        xfun::grep_sub('^[^<]*<[^>]+aria-labelledby[ ]*=[ ]*"([^"]+)".*$', '\\1', x)
+      ))
     }
   }
-  x = as.character(x)
+  x = wrap_asis(x, options)
   if (!out_format('latex') || inline) return(x)
   # latex output need the \end{kframe} trick
   options$results = 'asis'
@@ -500,7 +527,7 @@ sew.source = function(x, options, ...) {
 msg_wrap = function(message, type, options) {
   # when the output format is LaTeX, do not wrap messages (let LaTeX deal with wrapping)
   if (!length(grep('\n', message)) && !out_format(c('latex', 'listings', 'sweave')))
-    message = stringr::str_wrap(message, width = getOption('width'))
+    message = str_wrap(message, width = getOption('width'))
   knit_log$set(setNames(
     list(c(knit_log$get(type), paste0('Chunk ', options$label, ':\n  ', message))),
     type
@@ -621,12 +648,27 @@ sew.knit_embed_url = function(x, options = opts_chunk$get(), inline = FALSE, ...
   ))
 }
 
-add_html_caption = function(options, code) {
+add_html_caption = function(options, code, id = NULL) {
   cap = .img.cap(options)
-  if (cap == '') return(code)
+  if (cap == '' && !length(id)) return(code)
+
+  if (length(id)) {
+    alt = .img.cap(options, alt = TRUE)
+    if (cap == alt && cap != '') {
+      # both are the same, so insert cap with id
+      alttext = sprintf('<p class="caption" id="%s">%s</p>\n', id, cap)
+      # prevent a second insertion
+      cap = ''
+    } else {
+      alttext = sprintf('<p id="%s" hidden>%s</p>\n', id, alt)
+    }
+  } else alttext = ''
+
+  captext = if (cap == '') '' else sprintf('<p class="caption">%s</p>\n', cap)
+
   sprintf(
-    '<div class="figure"%s>\n%s\n<p class="caption">%s</p>\n</div>',
-    css_text_align(options$fig.align), code, cap
+    '<div class="figure"%s>\n%s\n%s%s</div>',
+    css_text_align(options$fig.align), code, captext, alttext
   )
 }
 
