@@ -216,7 +216,7 @@ quote_label = function(x) {
 # comment characters for various languages
 comment_chars = list(
   `#` = c('awk', 'bash', 'coffee', 'gawk', 'julia', 'octave', 'perl', 'powershell', 'python', 'r', 'ruby', 'sed', 'stan'),
-  '//' = c('asy', 'cc', 'csharp', 'd3', 'dot', 'fsharp', 'go', 'groovy', 'java', 'js', 'node', 'Rcpp', 'sass', 'scala'),
+  '//' = c('asy', 'cc', 'csharp', 'd3', 'dot', 'fsharp', 'go', 'groovy', 'java', 'js', 'node', 'Rcpp', 'sass', 'scss', 'scala'),
   `%` = c('matlab', 'tikz'),
   `/* */` = c('c', 'css'),
   `* ;` = c('sas'),
@@ -375,9 +375,6 @@ print.inline = function(x, ...) {
     if (nrow(x$location)) {
       cat(sprintf('  |  %s  #%s:%s', x$code, x$location[, 1], x$location[, 2]), sep = '\n')
     }
-  } else if (nrow(x$location) == 0 && xfun::check_old_package('autonewsmd', '0.0.4')) {
-    # TODO: remove this hack after https://github.com/kapsner/autonewsmd/pull/6
-    cat('  ordinary text without R code\n')
   }
 }
 
