@@ -1,4 +1,4 @@
-# graphics devices in base R, plus those in Cairo, cairoDevice, tikzDevice
+# graphics devices in base R, plus those in Cairo, tikzDevice
 auto_exts = c(
   bmp = 'bmp', postscript = 'eps', pdf = 'pdf', png = 'png', svg = 'svg',
   jpeg = 'jpeg', pictex = 'tex', tiff = 'tiff', win.metafile = 'wmf',
@@ -11,8 +11,6 @@ auto_exts = c(
   CairoJPEG = 'jpeg', CairoPNG = 'png', CairoPS = 'eps', CairoPDF = 'pdf',
   CairoSVG = 'svg', CairoTIFF = 'tiff',
 
-  Cairo_pdf = 'pdf', Cairo_png = 'png', Cairo_ps = 'eps', Cairo_svg = 'svg',
-
   svglite = 'svg', gridSVG = 'svg',
 
   ragg_png = 'png',
@@ -88,11 +86,6 @@ dev_get = function(dev, options = opts_current$get(), dpi = options$dpi[1]) {
     CairoPDF = load_device('CairoPDF', 'Cairo'),
     CairoSVG = load_device('CairoSVG', 'Cairo'),
 
-    Cairo_pdf = load_device('Cairo_pdf', 'cairoDevice'),
-    Cairo_png = load_device('Cairo_png', 'cairoDevice'),
-    Cairo_ps = load_device('Cairo_ps', 'cairoDevice'),
-    Cairo_svg = load_device('Cairo_svg', 'cairoDevice'),
-
     svglite = load_device('svglite', 'svglite'),
 
     # similar to load_device(), but the `dpi` argument is named `res`
@@ -210,14 +203,16 @@ patch_tikz_tex = function(path) {
 # filter the dev.args option
 get_dargs = function(dargs, dev) {
   if (length(dargs) == 0) return()
-  if (is.list(dargs) && all(sapply(dargs, is.list))) {
+  nms = names(dargs)
+  if (is.list(dargs) && all(sapply(dargs, is.list)) && length(nms) &&
+      (dev %in% nms || all(nms %in% names(auto_exts)))) {
     # dev.args is list(dev1 = list(arg1 = val1, ...), dev2 = list(arg2, ...))
     dargs = dargs[[dev]]
   }
   dargs
 }
 
-# this is mainly for Cairo and cairoDevice
+# this is mainly for Cairo
 load_device = function(name, package, dpi = NULL) {
   dev = getFromNamespace(name, package)
   # dpi is for bitmap devices; units must be inches!
@@ -378,7 +373,7 @@ plot_crop = function(x, quiet = TRUE) {
   is_pdf = grepl('[.]pdf$', x, ignore.case = TRUE)
   x2 = x
   x = path.expand(x)
-  if (is_pdf && !has_utility('pdfcrop') && !has_utility('ghostscript')) return(x2)
+  if (is_pdf && !has_utility('pdfcrop')) return(x2)
 
   if (!quiet) message('cropping ', x)
   if (is_pdf) {
